/**
	$Id: ExportedProperty.java 4898 2009-04-21 09:29:09Z martin $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.plugins;

import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Unit;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.formatter.Formatter;

/**

 @author nicklas
 @version 2.2
 @base.modified $Date: 2009-04-21 11:29:09 +0200 (Tue, 21 Apr 2009) $
 */
public class ExportedProperty
{
	public final String name;
	public final String title;
	public final AnnotationType annotationType;
	public final Unit unit;
	public final Formatter formatter;
	public final DataLoader dataloader;
	
	public static ExportedProperty parse(DbControl dc, String column, boolean annotatable, Formatter formatter)
	{
		return parse(dc, column, null, annotatable, formatter, true);
	}

	/**
		@since 2.8
	*/
	public static ExportedProperty parse(DbControl dc, String column, String columnPrefix, boolean annotatable, Formatter formatter)
	{
		return parse(dc, column, columnPrefix, annotatable, formatter, true);
	}
	
	/**
		@since 2.9
	*/
	public static ExportedProperty parse(DbControl dc, String column, String columnPrefix, 
		boolean annotatable, Formatter formatter, boolean sameUnits)
	{
		return parse(dc, column, columnPrefix, annotatable, formatter, sameUnits, null);
	}
	
	/**
	 	@since 2.12
	*/
	@SuppressWarnings("unchecked")
	public static ExportedProperty parse(DbControl dc, String column, String columnPrefix, 
			boolean annotatable, Formatter formatter, boolean sameUnits, DataLoader dataloader)
	{
		String[] p = column.split(":");
		String name = p[0];
		String title = p.length > 1 ? p[1] : p[0];
		AnnotationType at = null;
		Unit unit = null;
		String columnSuffix = null;;
		if (name.startsWith("#") && annotatable) // && itemType != Item.SPOTDATA)
		{
			// Annotation: properties[i] = #annotationTypeId
			int annotationTypeId = Values.getInt(name.substring(1));
			at = AnnotationType.getById(dc, annotationTypeId);
			name = "annotationtype_"+annotationTypeId;
			if (sameUnits && at.supportUnits()) 
			{
				unit = at.getDefaultUnit();
				if (unit != null) columnSuffix = " (" + unit.getDisplaySymbol() + ")";
			}
		}
		if (columnPrefix != null) title = columnPrefix + title;
		if (columnSuffix != null) title += columnSuffix;
		return new ExportedProperty(name, title, at, unit, formatter, dataloader);
	}

	public ExportedProperty(String name, String title, AnnotationType annotationType, Formatter formatter)
	{
		this(name, title, annotationType, annotationType.getDefaultUnit(), formatter);
	}
	
	/**
		@since 2.9
	*/
	public ExportedProperty(String name, String title, 
		AnnotationType annotationType, Unit unit, Formatter formatter)
	{
		this(name, title, annotationType, unit, formatter, null);		
	}
	
	public ExportedProperty(String name, String title, 
			AnnotationType annotationType, Unit unit, Formatter formatter, DataLoader dataloader)
	{
		this.name = name;
		this.title = title;
		this.annotationType = annotationType;
		this.formatter = formatter;
		this.unit = unit;
		this.dataloader = dataloader;
	}
}
