/**
	$Id: ItemQueryLoader.java 4900 2009-04-21 12:54:42Z martin $

	Copyright (C) 2009 Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.plugins;

import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.InvalidUseOfNullException;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Type;

/**
	A class that uses a given query to load data. The query is filtered
	on a given parameter.
	
	@author martin
	@version 2.12
	@base.modified $Date: 2009-04-21 14:54:42 +0200 (Tue, 21 Apr 2009) $
*/
public class ItemQueryLoader 
	implements DataLoader<BasicItem<?>>
{
	private final ItemQuery<?> query;
	private final String parameterName;
	
	/**	 	
	  	@param query Query to use when loading data
	  	@param parameterName Name of the parameter that should be used to filter the query.
	 */
	public ItemQueryLoader(ItemQuery<?> query, String parameterName)
	{
		if (query == null) throw new InvalidUseOfNullException("query");
		if (parameterName == null) throw new InvalidUseOfNullException("parameterName");
		this.query = query;
		this.parameterName = parameterName;
	}

	@Override
	public Object getData(ExportedProperty exportedProperty, BasicItem<?> item)
		throws Exception
	{
		if (item == null) return null;
		DbControl dc = item.getDbControl();
		query.setParameter(parameterName, item.getId(), Type.INT);
		return query.list(dc);
	}
}
