/*
	$Id: BioMaterialData.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Set;

/**
	This class is the root class for biomaterials.

	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.BioMaterial
	@see <a href="../../../../../../html/developerdoc/api/data_api.biomaterials.html">Biomaterials overview</a>
	@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
	@hibernate.class table="`BioMaterials`" lazy="false" discriminator-value="-1"
	@hibernate.discriminator column="`discriminator`" type="int"
*/
public abstract class BioMaterialData
	extends AnnotatedData
{

	public BioMaterialData()
	{}
	
	/**
		The maximum length of the external id that can be stored in the database.
		@see #setExternalId(String)
	*/
	public static final int MAX_EXTERNAL_ID_LENGTH = 255;
	private String externalId;
	/**
		Get the external id for the biomaterial
		@hibernate.property column="`external_id`" type="string" length="255" not-null="false"
	*/
	public String getExternalId()
	{
		return externalId;
	}
	public void setExternalId(String externalId)
	{
		this.externalId = externalId;
	}

	private Set<MeasuredBioMaterialData> children;
	/**
		This is the inverse end. NOTE! This collection will not 
		contained pooled children or hybridizations.
		@see MeasuredBioMaterialData#getParent()
		@since 2.9
		@hibernate.set inverse="true" lazy="true"
		@hibernate.collection-key column="`parent_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.MeasuredBioMaterialData"
	*/
	Set<MeasuredBioMaterialData> getChildren()
	{
		return children;
	}
	void setChildren(Set<MeasuredBioMaterialData> children)
	{
		this.children = children;
	}

	private Set<BioMaterialListData> lists;
	/**
		This is the inverse end. 
		NOTE! We do not map with inverse="true" since we want automatic 
		deletion from the join table when a biomaterial is deleted.
		@see BioMaterialListData#getBioMaterials()
		@since 2.10
		@hibernate.set lazy="true" table="`BioMaterialListMembers`"
		@hibernate.collection-key column="`biomaterial_id`"
		@hibernate.collection-many-to-many column="`list_id`" class="net.sf.basedb.core.data.BioMaterialListData"
	*/
	Set<BioMaterialListData> getBioMaterialLists()
	{
		return lists;
	}
	void setBioMaterialLists(Set<BioMaterialListData> lists)
	{
		this.lists = lists;
	}
	
}
