/*
	$Id: OwnableData.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005, 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

/**
	An ownable item is an item which has an owner. The owner is 
	represented as a required association to a {@link UserData}
	object.
	<p>
	This interface defines Hibernate database mappings for the
	<code>owner</code> property to the database column with the
	same name. If a subclass wants to map the property to another column,
	it should override the {@link #getOwner()} method and add a
	Hibernate tag in the comment.
	<p>
	
	<b>Reference implementation</b><br>
	<pre class="code">
private UserData owner;
public UserData getOwner()
{
   return owner;
}
public void setOwner(UserData owner)
{
   this.owner = owner;
}
</pre>

	@author Nicklas
	@version 2.0
	@see OwnedData
	@see net.sf.basedb.core.Ownable
	@see <a href="../../../../../../html/developerdoc/api/data_api.authentication.html">Authentication overview</a>
	@see <a href="../../../../../../html/developerdoc/api/data_api.basic.html">Basic classes and interfaces</a>
*/
public interface OwnableData
	extends IdentifiableData
{
	/**
		Get the {@link UserData} that is the owner of the item.
		@hibernate.many-to-one column="`owner`" not-null="true" outer-join="false"
	*/
	public UserData getOwner();

	/**
		Change the owner of the item. Null is not allowed.
	*/
	public void setOwner(UserData owner);
}
