/*
	$Id: PlateEventData.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Date;

/**
	This class holds information about a plate event.

	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.PlateEvent
	@see <a href="../../../../../../html/developerdoc/api/data_api.plates.html">Array LIMS - Plates overview</a>
	@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
	@hibernate.class table="`PlateEvents`" lazy="false"
*/
public class PlateEventData
	extends BasicData
{

	public PlateEventData()
	{}
	
	private PlateData plate;
	/**
		The plate this event belongs to.
		@see PlateData#getEvents()
		@hibernate.many-to-one column="`plate_id`" not-null="true" outer-join="false" update="false"
	*/
	public PlateData getPlate()
	{
		return plate;
	}
	public void setPlate(PlateData plate)
	{
		this.plate = plate;
	}

	private PlateEventTypeData plateEventType;
	/**
		The event type of this event. It must belong to the same plate type as the plate.
		@see PlateData#getPlateType()
		@see PlateEventTypeData#getPlateType()
		@hibernate.many-to-one column="`plateeventtype_id`" not-null="true" outer-join="false" update="false"
	*/
	public PlateEventTypeData getPlateEventType()
	{
		return plateEventType;
	}
	public void setPlateEventType(PlateEventTypeData plateEventType)
	{
		this.plateEventType = plateEventType;
	}

	private ProtocolData protocol;
	/**
		The protocol used for the event.
		@hibernate.many-to-one column="`protocol_id`" not-null="false" outer-join="false"
	*/
	public ProtocolData getProtocol()
	{
		return protocol;
	}
	public void setProtocol(ProtocolData protocol)
	{
		this.protocol = protocol;
	}
	
	private UserData user;
	/**
		Get the user that is responsible for this event, typically this is the user that
		entered the information in the database.
		@hibernate.many-to-one column="`user_id`" not-null="false" outer-join="false" update="false"
	*/
	public UserData getUser()
	{
		return user;
	}
	public void setUser(UserData user)
	{
		this.user = user;
	}
	
	private Date entryDate;
	/**
		Get the date this event was added to the database.
		@hibernate.property column="`entry_date`" type="date" not-null="true" update="false"
	*/
	public Date getEntryDate()
	{
		return entryDate;
	}
	public void setEntryDate(Date entryDate)
	{
		this.entryDate = entryDate;
	}

	private Date eventDate;
	/**
		Get the date this event happened in the lab.
		@hibernate.property column="`event_date`" type="date" not-null="false"
	*/
	public Date getEventDate()
	{
		return eventDate;
	}
	public void setEventDate(Date eventDate)
	{
		this.eventDate = eventDate;
	}
	
	private HardwareData hardware;
	/**
	 	Get the hardware used in this event.
	 	@hibernate.many-to-one column="`hardware_id`" not-null="false" outer-join="false"
	*/
	public HardwareData getHardware()
	{
		return this.hardware;
	}
	public void setHardware(HardwareData hardware)
	{
		this.hardware = hardware;
	}

	/**
		The maximum length of the comment about this event.
	*/
	public static final int MAX_COMMENT_LENGTH = 65535;
	private String comment;
	/**
		Get the comment about this event.
		@hibernate.property column="`comment`" type="text" not-null="false"
	*/
	public String getComment()
	{
		return comment;
	}
	public void setComment(String comment)
	{
		this.comment = comment;
	}



}
