/*
	$Id: PlateGeometryData.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Date;
import java.util.Set;

/**
	This class holds information about a plate geometry.

	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.PlateGeometry
	@see <a href="../../../../../../html/developerdoc/api/data_api.plates.html">Array LIMS - Plates overview</a>
	@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
	@hibernate.class table="`PlateGeometries`" lazy="false"
*/
public class PlateGeometryData
	extends BasicData
	implements NameableData, RemovableData, RegisteredData
{

	public PlateGeometryData()
	{}
	
	/*
		From the NameableData interface
		-------------------------------------------
	*/
	private String name;
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}
	private String description;
	public String getDescription()
	{
		return description;
	}
	public void setDescription(String description)
	{
		this.description = description;
	}
	// -------------------------------------------
	/*
		From the RemovableData interface
		-------------------------------------------
	*/
	private boolean removed;
	public boolean isRemoved()
	{
		return removed;
	}
	public void setRemoved(boolean removed)
	{
		this.removed = removed;
	}
	// -------------------------------------------
	/*
		From the RegisteredData interface
		-------------------------------------------
	*/
	private Date entryDate;
	public Date getEntryDate()
	{
		return entryDate;
	}
	public void setEntryDate(Date entryDate)
	{
		this.entryDate = entryDate;
	}
	// -------------------------------------------

	private int rows;
	/**
		The number of rows in this geometry.
		@hibernate.property column="`rows`" type="int" not-null="true" update="false"
	*/
	public int getRows()
	{
		return rows;
	}
	public void setRows(int rows)
	{
		this.rows = rows;
	}
	
	private int columns;
	/**
		The number of rows in this geometry.
		@hibernate.property column="`columns`" type="int" not-null="true" update="false"
	*/
	public int getColumns()
	{
		return columns;
	}
	public void setColumns(int columns)
	{
		this.columns = columns;
	}
	
	private Set<PlateTypeData> plateTypes;
	/**
		This is the invers end.
		@see PlateTypeData#getPlateGeometry()
		@since 2.9
		@hibernate.set lazy="true" inverse="true" cascade="delete"
		@hibernate.collection-key column="`plategeometry_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.PlateTypeData"
	*/
	Set<PlateTypeData> getPlateTypes()
	{
		return plateTypes;
	}
	void setPlateTypes(Set<PlateTypeData> plateTypes)
	{
		this.plateTypes = plateTypes;
	}
}
