/*
	$Id: CountExpression.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.Collection;
import java.util.Collections;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.BaseException;

/**
	Count the number of values: COUNT(e)

	@author Nicklas
	@version 2.0
	@see Aggregations#count(Expression, boolean)
	@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/
public class CountExpression
	implements Expression 
{
	private final Expression e;
	private final boolean distinct;

	CountExpression(Expression e, boolean distinct)
	{
		this.e = e;
		this.distinct = distinct;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		return "COUNT(" + (distinct ? "DISTINCT " : "") + (e == null ? "*" : e.toQl(query, dc)) + ")";
	}
	/**
		@return The expression to count
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return e == null ? null : Collections.singleton(e);
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return "COUNT(" + (distinct ? "DISTINCT " : "") + (e == null ? "*" : e ) + ")";
	}
	// -------------------------------------------
}

