/*
	$Id: DivideExpression.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Samuel Andersson, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.Arrays;
import java.util.Collection;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.HibernateUtil;

/**
	Divide one expression by another: e1 / e2.

	@author Nicklas
	@version 2.0
	@see Expressions#divide(Expression, Expression)
	@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/
public class DivideExpression
	implements Expression 
{
	private final Expression e1;
	private final Expression e2;

	DivideExpression(Expression e1, Expression e2)
	{
		assert e1 != null : "e1 == null";
		assert e2 != null : "e2 == null";
		this.e1 = e1;
		this.e2 = e2;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		String numerator = e1.toQl(query, dc);
		String denominator = e2.toQl(query, dc);
		if (HibernateUtil.getDbEngine().checkForInvalidNumberOperation())
		{
			return "(CASE WHEN " + denominator + "=0 THEN null ELSE " + numerator + "/" + denominator + " END)";
		}
		else
		{
			return "(" + numerator + " / " + denominator + ")";
		}
	}
	/**
		@return The expressions to divide
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return Arrays.asList(e1, e2);
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return "(" + e1 + " / " + e2 + ")";
	}
	// -------------------------------------------
}

