/*
	$Id: SqlColumnExpression.java 4516 2008-09-11 20:09:17Z jari $

	Copyright (C) 2006 Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.Collection;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.HibernateUtil;
import net.sf.basedb.core.ReporterList;

/**
	An expression representing the column in a
	database tabe: alias.column.
	<p>
	Throws an <code>UnsupportedOperationException</code> if 
	{@link Query#getQueryType()} isn't {@link QueryType#SQL}.

	@author Nicklas
	@version 2.0
	@see Dynamic#score(ReporterList)
	@base.modified $Date: 2008-09-11 22:09:17 +0200 (Thu, 11 Sep 2008) $
*/
public class SqlColumnExpression
	implements Expression
{
	private final String alias;
	private final String column;

	SqlColumnExpression(String alias, String column)
	{
		this.alias = alias;
		this.column = column;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		if (query.getQueryType() == QueryType.SQL)
		{
			return (alias == null ? query.getRootAlias() : alias) + "." + HibernateUtil.quote(column);
		}
		else
		{
			throw new UnsupportedOperationException("'sqlColumn' operation not supported by "+query.getQueryType()+" query.");
		}
	}
	/**
		@return Always null
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return null;
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return (alias == null ? "" : alias + ".") + column;
	}
	// -------------------------------------------
}
