/*
	$Id: SqlResult.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.sql.SQLException;

/**
	This interface represents a single row in the results of
	a {@link SqlQuery}.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/
public interface SqlResult
{

	/**
		Get the value of the specified column as a short.
		@param index The index of the column, starting at 1
		@return The value, or 0 if the database contains null
		@throws SQLException If an error occurs
	*/
	public short getShort(int index)
		throws SQLException;

	/**
		Get the value of the specified column as an integer.
		@param index The index of the column, starting at 1
		@return The value, or 0 if the database contains null
		@throws SQLException If an error occurs
	*/
	public int getInt(int index)
		throws SQLException;

	/**
		Get the value of the specified column as a float.
		@param index The index of the column, starting at 1
		@return The value, or 0 if the database contains null
		@throws SQLException If an error occurs
	*/
	public float getFloat(int index)
		throws SQLException;

	/**
		Get the value of the specified column as a string.
		@param index The index of the column, starting at 1
		@return The string value
		@throws SQLException If an error occurs
	*/
	public String getString(int index)
		throws SQLException;
		
	/**
		Get the value of the specified column as an object.
		@param index The index of the column, starting at 1
		@return The string value
		@throws SQLException If an error occurs
	*/
	public Object getObject(int index)
		throws SQLException;
		
		
	// TODO - more data types; Date, etc. 
	// TODO - wasNull ?

}
