/**
	$Id: SocketSignalTransporter.java 4516 2008-09-11 20:09:17Z jari $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.signal;

import java.net.Socket;
import java.net.URI;

import net.sf.basedb.util.SocketUtil;

/**

	@author nicklas
	@version 2.6
	@base.modified $Date: 2008-09-11 22:09:17 +0200 (Thu, 11 Sep 2008) $
*/
public class SocketSignalTransporter
	extends AbstractSignalTransporter
{

	/**
		Log signals processing.
	*/
	private static final org.apache.log4j.Logger logger = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.core.signal.SocketSignalTransporter");
	
	/**
		Create a new socket signal transporter. Before it can be used it must
		be initialised with {@link #init(String)}.
	*/
	public SocketSignalTransporter()
	{}
	
	/*
		From the SignalTransporter interface
		-------------------------------------------
	*/
	/**
		Connect to the remote host and send the signal.
	*/
	public void send(Signal signal)
	{
		logger.info("Sending signal " + signal.getId() + " to " + getGlobalSignalId());
		URI uri = getSignalURI();
		String host = uri.getHost();
		int port = uri.getPort();
		String message = generateSignalMessage(signal);
		if (uri.getFragment() != null) message += "#" + uri.getFragment(); // For proxy support
		logger.debug("The message is: " + message);
		Socket s = null;
		try
		{
			s = new Socket(host, port);
			SocketUtil.send(s, message, true);
		}
		catch (Exception ex)
		{
			logger.error("Error sending signal " + signal.getId() + " to " + getGlobalSignalId(), ex);
			throw new SignalException(ex);
		}
		finally
		{
			if (s != null) SocketUtil.close(s);
		}
	}
	// -------------------------------------------

}
