/**
	$Id: JobInfo.java 4515 2008-09-11 20:08:14Z jari $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.jobagent;

import java.util.List;
import java.util.Map;

import net.sf.basedb.core.Job;

/**
	Hold information about a job that is running on a job agent.

	@author nicklas
	@version 2.6
	@base.modified $Date: 2008-09-11 22:08:14 +0200 (Thu, 11 Sep 2008) $
*/
public class JobInfo
{
	private int jobId;
	private Job.ExecutionTime slot;
	
	/**
		Create a new JobInfo object. 
		@param job
		@param slot
	*/
	public JobInfo(Job job, Job.ExecutionTime slot)
	{
		this.jobId = job.getId();
		this.slot = slot;
	}

	/**
		Create a new JobInfo object using information from the answer to a
		{@link JobAgentConnection} request.
	*/
	JobInfo(int jobId, Map<String, List<String>> headers)
	{
		this.jobId = jobId;
		String slotHeader = getValue(headers, "Job." + jobId + ".slot");
		this.slot = slotHeader == null ? 
				null : Job.ExecutionTime.valueOf(slotHeader);	
	}
	
	/**
		Get the ID of the job.
	*/
	public int getJobId()
	{
		return jobId;
	}
	
	/**
		Get the slot the job is using on the job agent.
	*/
	public Job.ExecutionTime getSlot()
	{
		return slot;
	}
	
	/**
		Convert the information to a string that is suitable for sening as an
		answer to a {@link JobAgentConnection} object.
	*/
	@Override
	public String toString()
	{
		StringBuilder sb = new StringBuilder();
		sb.append("Job:").append(jobId).append("\n");
		if (slot != null) 
		{
			sb.append("Job.").append(jobId).append(".slot:").append(slot.name()).append("\n");
		}
		return sb.toString();
	}

	private String getValue(Map<String, List<String>> headers, String header)
	{
		List<String> values = headers.get(header);
		String value = null;
		if (values != null && values.size() > 0)
		{
			value = values.get(0);
		}
		return value;
	}

	@Override
	public boolean equals(Object other)
	{
		if (this == other) return true;
		if (!(other instanceof JobInfo)) return false;
		return this.jobId == ((JobInfo)other).jobId;
	}
	
	@Override
	public int hashCode()
	{
		return jobId;
	}
	
}
