/**
	$Id: IndexedCacheKey.java 4768 2009-02-18 13:26:21Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.cache;

/**
	Cache key implementation that bundles another cache key
	with an index. This type of key is used when moving in
	the reverse direction from a raw bioassay to hybridization
	and labeled extracts. The reason is that in this case only
	the labeled extracts that belong to the same array index
	as the raw bioassay should be loaded. Thus, we also need
	different cache entries for different array indexes.

	@author Nicklas
	@version 2.10
	@base.modified $Date: 2009-02-18 14:26:21 +0100 (Wed, 18 Feb 2009) $
*/
public class IndexedCacheKey
{
	private final Object cacheKey;
	private final int index;
	
	public IndexedCacheKey(Object cacheKey, int index)
	{
		this.cacheKey = cacheKey;
		this.index = index;
	}

	/**
		Get the original cache key passed to the constructor.
	*/
	public Object unwrap()
	{
		return cacheKey;
	}
	
	@Override
	public boolean equals(Object obj)
	{
		if (this == obj) return true;
		if (obj == null) return false;
		if (obj.getClass() != this.getClass()) return false;
		IndexedCacheKey other = (IndexedCacheKey)obj;
		return this.index == other.index && this.cacheKey.equals(other.cacheKey);
	}

	@Override
	public int hashCode()
	{
		return cacheKey.hashCode() * 7 + index;
	}
	
}
