/*
	$Id: TestGlobalDefaultSetting.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;

public class TestGlobalDefaultSetting
{

	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing global default setting");
		write_header();
		// Standard tests: create, load, list
		int id = test_create("fontsize", "9");
		test_duplicate(id);
		test_load(id);
		test_list(-1);

		// Extra tests:

		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		// Standard test: Delete
		test_delete(id);
		write("++Testing global default setting "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(String name, String value)
	{
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			GlobalDefaultSetting gds = GlobalDefaultSetting.getNew(dc, name, value);
			dc.saveItem(gds);
			dc.commit();
			id = gds.getId();
			write_item(0, gds);
			write("--Create global default setting OK");
		}
		catch (Throwable ex)
		{
			write("--Create global default setting FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			GlobalDefaultSetting gds = GlobalDefaultSetting.getById(dc, id);
			write_item(0, gds);
			write("--Load global default setting OK");
		}
		catch (Throwable ex)
		{
			write("--Load global default setting FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<GlobalDefaultSetting> l = GlobalDefaultSetting.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List global default settings OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List global default settings FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			GlobalDefaultSetting gds = GlobalDefaultSetting.getById(dc, id);
			dc.deleteItem(gds);
			dc.commit();
			write("--Delete global default setting OK");
		}
		catch (Throwable ex)
		{
			write("--Delete global default setting FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName\tValuet");
			write("-- \t-- \t----\t------");
		}
	}
	static void write_item(int i, GlobalDefaultSetting gds)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+gds.getId()+"\t"+gds.getName()+"\t"+
			gds.getValue());
	}
	static void write(String message)
	{
		System.out.println(message);
	}
	
	static void test_duplicate(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			GlobalDefaultSetting gds = GlobalDefaultSetting.getById(dc, id);
			GlobalDefaultSetting duplicate = GlobalDefaultSetting.getNew(dc, gds.getName(), gds.getValue());
			dc.saveItem(duplicate);
			dc.commit();
			dc.deleteItem(duplicate);
			dc.commit();
			throw new BaseException("Expected DatabaseException");
		}
		catch (DatabaseException ex)
		{
			write("--Duplicate global default setting OK");
		}
		catch (Throwable ex)
		{
			write("--Duplicate global default setting FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
}
