/*
	$Id: TestJarClassLoader.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.plugin.Plugin;
import net.sf.basedb.core.plugin.About;
import net.sf.basedb.util.JarClassLoader;
import java.net.URL;
import java.io.InputStreamReader;
import java.io.BufferedReader;

public class TestJarClassLoader
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing Jar class loader");
		
		test_load("JarPlugin.jar", "JarPlugin");
		test_resource("JarPlugin.jar", "JarPluginManifest.txt");
		test_unload("JarPlugin.jar");
		
		write("++Testing Jar class loader "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static void test_load(String jarPath, String className)
	{
		try
		{
			ClassLoader cl = JarClassLoader.getInstance(jarPath);
			Class c = cl.loadClass(className);
			Object o = c.newInstance();
			write_item(o);
			if (o instanceof Plugin)
			{
				Plugin p = (Plugin)o;
				write_item(p);
			}
			write("--Load class OK");
		}
		catch (Throwable ex)
		{
			write("--Load class FAILED");
			ex.printStackTrace();
			ok = false;
		}
	}
	
	static void test_resource(String jarPath, String name)
	{
		try
		{
			ClassLoader cl = JarClassLoader.getInstance(jarPath);
			URL resource = cl.getResource(name);
			BufferedReader in = new BufferedReader(new InputStreamReader(resource.openStream()));
			String line = in.readLine();
			StringBuilder content = new StringBuilder();
			while (line != null)
			{
				content.append(line);
				line = in.readLine();
			}
			in.close();
			
			if (!TestUtil.getSilent())
			{
				write("Resource URL: "+resource);
				write("Resource content: "+content.toString());
			}
			write("--Load resource OK");
		}
		catch (Throwable ex)
		{
			write("--Load resource FAILED");
			ex.printStackTrace();
			ok = false;
		}
	}

	
	static void test_unload(String jarPath)
	{
		try
		{
			JarClassLoader.unload(jarPath);
			if (JarClassLoader.exists(jarPath))
			{
				throw new Exception("Class loader for '"+jarPath+"' was not unloaded.");
			}
			write("--Unload class loader OK");
		}
		catch (Throwable ex)
		{
			write("--Unload class loader FAILED");
			ex.printStackTrace();
			ok = false;
		}
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}
	
	static void write_item(Object o)
	{
		if (!TestUtil.getSilent())
		{
			write("Object '"+o+"' of class '"+o.getClass().getName()+"' loaded");
		}
	}
	
	static void write_item(Plugin p)
	{
		if (!TestUtil.getSilent())
		{
			About a = p.getAbout();
			Plugin.MainType t = p.getMainType();
			write(a.getName()+"\t"+t+"\t"+a.getDescription());
		}
	}

}

