/*
 $Id: TestPlatform.java 4514 2008-09-11 20:07:12Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	 BASE is free software; you can redistribute it and/or
	 modify it under the terms of the GNU General Public License
	 as published by the Free Software Foundation; either version 3
	 of the License, or (at your option) any later version.
	
	 BASE is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 GNU General Public License for more details.
	
	 You should have received a copy of the GNU General Public License
	 along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.FileType;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.PlatformFileType;
import net.sf.basedb.core.PlatformVariant;
import net.sf.basedb.core.RawDataTypes;
import net.sf.basedb.core.SystemItems;

public class TestPlatform
{

	static boolean ok = true;
	
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing platform");
		write_header();
		// Standard tests: create, load and list

		int id1 = test_create("Db and file", "test.db.and.file", false, 0, null);
		int id2 = test_create("File-only", "test.file.only", true, 1, null);
		int id3 = test_create("Locked to genepix", "test.locked.genepix", false, 0, "genepix");
		test_list();

		// Variants
		int idVariant1 = test_create_variant(id1, "Variant of db and file", "test.db.and.file.v1", null, 0, null);
		int idVariant2 = test_create_variant(id2, "Variant #1 file-only", "test.file.only.v1", null, 0, null);
		int idVariant2b = test_create_variant(id2, "Variant #2 file-only", "test.file.only.v2", null, 0, null);
		int idVariant3 = test_create_variant(id3, "Locked to agilent", "test.locked.agilent", false, 0, "agilent");
		test_list_variants(id1, 1);
		
		// File types
		int fileTypeId = TestDataFileType.test_create("test.datafile", "test.datafile", Item.RAWBIOASSAY, SystemItems.getId(FileType.RAW_DATA));
		int celId = TestDataFileType.get_id_for_external("affymetrix.cel");
		int cdfId = TestDataFileType.get_id_for_external("affymetrix.cdf");
		test_add_filetype(id2, 0, celId, true);
		test_add_filetype(id2, idVariant2, cdfId, false);
		test_add_filetype(id2, idVariant2b, fileTypeId, false);
		test_list_filetypes(id2, 0, false, 3);
		test_list_filetypes(id2, 0, true, 1);
		test_list_filetypes(id2, idVariant2, false, 2);
		test_list_filetypes(id2, idVariant2, true, 1);
		
		// Standard test: Delete
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_delete_variant(idVariant1);
		test_delete(id1);
		test_delete(id2);
		test_delete(id3);
		TestDataFileType.test_delete(fileTypeId);
		write("++Testing platform "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(String name, String systemId, boolean fileOnly, int channels, String rawDataType)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.PLATFORM)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			
			Platform p = fileOnly ? 
				Platform.getNew(dc, systemId, channels) :
				Platform.getNew(dc, systemId, RawDataTypes.getRawDataType(rawDataType));
			p.setName(name);
			dc.saveItem(p);
			dc.commit();
			id = p.getId();
			write_item(0, p);
			write("--Create platform OK");
		}
		catch (Throwable ex)
		{
			write("--Create platform FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	
	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Platform p = Platform.getById(dc, id);
			write_item(0, p);
			write("--Load platform OK");
		}
		catch (Throwable ex)
		{
			write("--Load platform FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<Platform> l = Platform.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			write("--List platforms OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List platforms FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Platform l = Platform.getById(dc, id);
			dc.deleteItem(l);
			dc.commit();
			write("--Delete platform OK");
		}
		catch (Throwable ex)
		{
			write("--Delete platform FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static int test_get(String externalId)
	{
		DbControl dc = null;
		int platformId = 0;
		try
		{
			dc = TestUtil.getDbControl();
			Platform p = Platform.getByExternalId(dc, externalId);
			platformId = p.getId();
			write("--Get platform '"+externalId+"' OK");
		}
		catch (Throwable ex)
		{
			write("--Get platform '" + externalId + "' FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return platformId;
	}

	static int test_get_variant(String externalId)
	{
		DbControl dc = null;
		int variantId = 0;
		try
		{
			dc = TestUtil.getDbControl();
			PlatformVariant p = PlatformVariant.getByExternalId(dc, externalId);
			variantId = p.getId();
			write("--Get platform variant '"+externalId+"' OK");
		}
		catch (Throwable ex)
		{
			write("--Get platform variant '" + externalId + "' FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return variantId;
	}

	
	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tExternal ID\tFile only\tRaw data type");
			write("-- \t-- \t--------- \t-----------\t---------\t---------\t-------------");
		}
	}
	
	static void write_item(int i, Platform p)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.println(i+":\t"+p.getId()+"\t"+p.getName()+"\t"+
					p.getDescription() + "\t" + p.getExternalId() + "\t" + p.isFileOnly() + "\t" +
					p.getRawDataType());
		}
	}
	
	static void write_item(int i, PlatformVariant v)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.println(i+":\t"+v.getId()+"\t"+v.getName()+"\t"+
					v.getDescription() + "\t" + v.getExternalId() + "\t" + v.isFileOnly() + "\t" +
					v.getRawDataType() + "\t" + v.getPlatform());
		}
	}
	
	static void write_item(int i, PlatformFileType ft)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.println(i+":\t"+ft.getId()+"\t"+ft.getPlatform()+"\t"+
					ft.getVariant() + "\t" + ft.getDataFileType());
		}
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}

	static int test_create_variant(int platformId, String name, String systemId, Boolean fileOnly, int channels, String rawDataType)
	{
		if (platformId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.PLATFORMVARIANT)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			
			Platform p = Platform.getById(dc, platformId);
			
			PlatformVariant v = null;
			if (fileOnly == null)
			{
				v = PlatformVariant.getNew(dc, p, systemId);
			}
			else if (fileOnly == true)
			{
				v = PlatformVariant.getNew(dc, p, systemId, channels);
			}
			else
			{
				v = PlatformVariant.getNew(dc, p, systemId, RawDataTypes.getRawDataType(rawDataType));
			}
			v.setName(name);
			dc.saveItem(v);
			dc.commit();
			id = v.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(v);
			write_item(0, v);
			write("--Create platform variant OK");
		}
		catch (Throwable ex)
		{
			write("--Create platform variant FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void test_list_variants(int platformId, int expectedResults)
	{
		if (platformId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Platform p = Platform.getById(dc, platformId);
			ItemResultList<PlatformVariant> l = p.getVariants().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List platform variants OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List platform variants FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete_variant(int variantId)
	{
		if (variantId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PlatformVariant v = PlatformVariant.getById(dc, variantId);
			dc.deleteItem(v);
			dc.commit();
			write("--Delete platform variant OK");
		}
		catch (Throwable ex)
		{
			write("--Delete platform variant FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_add_filetype(int platformId, int variantId, int fileTypeId, boolean required)
	{
		if (platformId == 0 || fileTypeId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			
			Platform p = Platform.getById(dc, platformId);
			PlatformVariant v = variantId == 0 ? null : PlatformVariant.getById(dc, variantId);
			DataFileType fileType = DataFileType.getById(dc, fileTypeId);
			
			p.addFileType(fileType, required, v);
			
			dc.commit();
			write("--Add file type to platform OK");
		}
		catch (Throwable ex)
		{
			write("--Add file type to platform FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}	
	
	static void test_list_filetypes(int platformId, int variantId, boolean restrict, int expectedResults)
	{
		if (platformId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Platform p = Platform.getById(dc, platformId);
			PlatformVariant v = variantId == 0 ? null : PlatformVariant.getById(dc, variantId);
			ItemResultList<PlatformFileType> l = p.getFileTypes(v, restrict).list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List platform file types OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List platform file types FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	
}
