import java.util.Date;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.Quantity;

/*
	$Id $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

public class TestQuantity
{
	static boolean ok = true;
	
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing quantities");
		write_header();
		// Standard tests: create, load, list
		int id1 = test_create(true);
		int id2 = test_create(false);
		test_load(id1);
		test_list();

		// Standard test: Delete
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_delete(id1);
		test_delete(id2);
		write("++Testing quantities "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.QUANTITY)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Quantity q = Quantity.getNew(dc, "1 test unit");
			if (setAll)
			{
				q.setName("Test quantity");
				q.setDescription("Added at "+new Date());
			}
			dc.saveItem(q);
			dc.commit();
			id = q.getId();
			write_item(0, q);
			write("--Create quantity OK");
		}
		catch (Throwable ex)
		{
			write("--Create quantity FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Quantity q = Quantity.getById(dc, id);
			write_item(0, q);
			write("--Load quantity OK");
		}
		catch (Throwable ex)
		{
			write("--Load quantity FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<Quantity> l = Quantity.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			write("--List quantities OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List quantities FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Quantity q = Quantity.getById(dc, id);
			dc.deleteItem(q);
			dc.commit();
			write("--Delete quantity OK");
		}
		catch (Throwable ex)
		{
			write("--Delete quantity FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tReference unit\tSystem");
			write("-- \t-- \t--------- \t-----------\t--------------\t------");
		}
	}
	
	static void write_item(int i, Quantity q)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.println(i+":\t"+q.getId()+"\t"+q.getName()+"\t"+q.getDescription()+"\t"+
				q.getReferenceUnit() + "\t" + q.isSystemItem());
		}
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}
}
