/*
	$Id: TestScan.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;

import java.util.Date;

public class TestScan
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing scans");
		write_header();
		
		int hybridizationId = TestHybridization.test_create(0, false);
		int scannerId = TestHardware.test_create(SystemItems.getId(HardwareType.SCANNER), null, false);
		int protocolId = TestProtocol.test_create(SystemItems.getId(ProtocolType.SCANNING), null, false);
		
		// Standard tests: create, load, list
		int id = test_create(hybridizationId, scannerId, protocolId, true);
		int id2 = test_create(hybridizationId, 0, 0, false);
		
		test_load(id);
		test_list(-1);
		
		test_add_image(id);

		// Standard test: Delete
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_delete(id);
		test_delete(id2);
		
		TestHybridization.test_delete(hybridizationId);
		TestHardware.test_delete(scannerId);
		TestProtocol.test_delete(protocolId);
		write("++Testing scans "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(int hybridizationId, int scannerId, int protocolId, boolean setAll)
	{
		if (hybridizationId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.SCAN)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Scan s = Scan.getNew(dc, Hybridization.getById(dc, hybridizationId));
			if (setAll)
			{
				s.setName("Test scan");
				s.setDescription("Added at "+new Date());
			}
			if (scannerId != 0)
			{
				s.setScanner(Hardware.getById(dc, scannerId));
			}
			if (protocolId != 0)
			{
				s.setProtocol(Protocol.getById(dc, protocolId));
			}
			dc.saveItem(s);
			dc.commit();
			id = s.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(s);
			write_item(0, s);
			write("--Create scan OK");
		}
		catch (Throwable ex)
		{
			write("--Create scan FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Scan s = Scan.getById(dc, id);
			write_item(0, s);
			write("--Load scan OK");
		}
		catch (Throwable ex)
		{
			write("--Load scan FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<Scan> l = Scan.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List scans OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List scans FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Scan s = Scan.getById(dc, id);
			dc.deleteItem(s);
			dc.commit();
			write("--Delete scan OK");
		}
		catch (Throwable ex)
		{
			write("--Delete scan FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete_all()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<Scan> l = Scan.getQuery().list(dc);
			Trashcan.delete(dc.getSessionControl(), l, true, null);
			dc.commit();
			write("--Delete all scans OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--Delete all scans FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	
	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tHybridization\tScanner\tProtocol");
			write("-- \t-- \t--------- \t-----------\t-------------\t-------\t--------");
		}
	}
	static void write_item(int i, Scan s)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
		{
			System.out.println(i+":\t"+s.getId()+"\t"+s.getName()+"\t"+s.getDescription()+
				"\t"+s.getHybridization()+"\t"+s.getScanner()+"\t"+s.getProtocol());
		}
	}
	static void write_item(int i, Image img)
		throws BaseException
	{
		if (!TestUtil.getSilent())
		{
			System.out.println(i+":\t"+img.getId()+"\t"+img.getName()+"\t"+
				img.getDescription()+"\t"+img.getScan()+"\t"+
				img.getFormat()+"\t"+img.isPreview());
		}
	}
	static void write(String message)
	{
		System.out.println(message);
	}
	
	static void test_add_image(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Scan s = Scan.getById(dc, id);
			Image img = s.newImage();
			img.setFormat(Image.Format.JPEG);
			img.setPreview(true);
			img.setName("Image for "+s.getName());
			dc.saveItem(img);
			dc.commit();
			dc = TestUtil.getDbControl();
			dc.reattachItem(img);
			write_item(0, img);
			write("--Add image to scan OK");
		}
		catch (Throwable ex)
		{
			write("--Add image to scan FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
}
