/**
	$Id: UserTest.java 4789 2009-02-24 10:07:07Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.test.roles;

import net.sf.basedb.core.Annotatable;
import net.sf.basedb.core.Annotation;
import net.sf.basedb.core.AnnotationSet;
import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.BioMaterialEvent;
import net.sf.basedb.core.BioPlate;
import net.sf.basedb.core.BioSource;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.Extract;
import net.sf.basedb.core.File;
import net.sf.basedb.core.FileParameterType;
import net.sf.basedb.core.FileType;
import net.sf.basedb.core.Hybridization;
import net.sf.basedb.core.Include;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemParameterType;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.Label;
import net.sf.basedb.core.LabeledExtract;
import net.sf.basedb.core.PlateGeometry;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.PluginConfiguration;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.Project;
import net.sf.basedb.core.Quantity;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.RawDataType;
import net.sf.basedb.core.RawDataTypes;
import net.sf.basedb.core.Sample;
import net.sf.basedb.core.Scan;
import net.sf.basedb.core.StringParameterType;
import net.sf.basedb.core.SystemItems;
import net.sf.basedb.core.Unit;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Orders;
import net.sf.basedb.test.FileUtil;
import net.sf.basedb.test.PluginUtil;
import net.sf.basedb.test.TestUtil;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
	This class contains the tests for the regular user created
	by the root user.
	<ol>
	</ol>

	@author nicklas
	@version 2.0
	@base.modified $Date: 2009-02-24 11:07:07 +0100 (Tue, 24 Feb 2009) $
*/
public class UserTest
{
	public static boolean ok = true;
	
	/**
		Run the tests in this class.
	*/
	public static boolean run(boolean useBatchImporters)
	{
		DbControl dc = null;
		TestUtil.write("++Entering user test\n");
		try
		{
			TestUtil.login("user", "user", false);
			PluginDefinition bioSourceBatchImporter = null;
			PluginDefinition sampleBatchImporter = null;
			PluginDefinition extractBatchImporter = null;
			PluginDefinition labeledExtractBatchImporter = null;
			PluginDefinition hybridizationBatchImporter = null;
			PluginDefinition scanBatchImporter = null;
			PluginDefinition rawBioAssayBatchImporter = null;
			PluginDefinition annotationImporter = null;
			
			File batchImportFile_BioSource = null;
			File batchImportFile_Sample = null;
			File batchImportFile_Extract = null;
			File batchImportFile_LabeledExtract = null;
			File batchImportFile_Hybridization = null;
			File batchImportFile_Scan = null;
			File batchImportFile_RawBioAssay = null; 
			

			// Activate project
			dc = TestUtil.getDbControl();
			Project p = Util.findProject(dc, "Project A");
			dc.close();
			TestUtil.getSessionControl().setActiveProject(p);
			
			if (useBatchImporters)
			{
				// Preparation; uploading files and find plugins
				dc = TestUtil.getDbControl();
				bioSourceBatchImporter = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.batchimport.BioSourceImporter");
				sampleBatchImporter = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.batchimport.SampleImporter");
				extractBatchImporter = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.batchimport.ExtractImporter");
				labeledExtractBatchImporter = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.batchimport.LabeledExtractImporter");
				hybridizationBatchImporter = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.batchimport.HybridizationImporter");
				scanBatchImporter = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.batchimport.ScanImporter");
				rawBioAssayBatchImporter = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.batchimport.RawBioAssayImporter");
				annotationImporter = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.AnnotationFlatFileImporter");
				
				batchImportFile_BioSource = FileUtil.uploadFile(dc, "/batchimport", "biosource_out.txt", null);
				batchImportFile_Sample = FileUtil.uploadFile(dc, "/batchimport", "sample_out.txt", null);
				batchImportFile_Extract = FileUtil.uploadFile(dc, "/batchimport", "extract_out.txt", null);
				batchImportFile_LabeledExtract = FileUtil.uploadFile(dc, "/batchimport", "labeledextract_out.txt", null);
				batchImportFile_Hybridization = FileUtil.uploadFile(dc, "/batchimport", "hybridization_out.txt", null);
				batchImportFile_Scan = FileUtil.uploadFile(dc, "/batchimport", "scan_out.txt", null);
				batchImportFile_RawBioAssay = FileUtil.uploadFile(dc, "/batchimport", "rawbioassay_out.txt", null);
				dc.commit();
			}

			// Create biomaterials
			dc = TestUtil.getDbControl();
			Quantity time = Quantity.getById(dc, SystemItems.getId(Quantity.TIME));
			Unit hoursUnit = time.getUnit("hour");
			Platform genericPlatform = Platform.getByExternalId(dc, Platform.GENERIC);
			Platform affymetrixPlatform = Platform.getByExternalId(dc, Platform.AFFYMETRIX);
			BioPlate bioPlate = createBioPlate(dc, "Bioplate A", Util.findGeometry(dc, 16, 24));
			
			// Upload raw data files
			File rawData1 = FileUtil.uploadFile(dc, "/mouse", "genepix.mouse.v4.37k.00h.gpr", FileType.RAW_DATA);
			File rawData2 = FileUtil.uploadFile(dc, "/mouse", "genepix.mouse.v4.37k.24h.gpr", FileType.RAW_DATA);
			File rawData1DyeSwap = FileUtil.uploadFile(dc, "/mouse", "genepix.mouse.v4.37k.00h.dyeswap.gpr", FileType.RAW_DATA);
			File rawData2DyeSwap = FileUtil.uploadFile(dc, "/mouse", "genepix.mouse.v4.37k.24h.dyeswap.gpr", FileType.RAW_DATA);
			
			File affy1 = FileUtil.uploadFile(dc, "/affymetrix/E-TEST-1.ebi.ac.uk", "jos1761.cel.tar.bz2", FileType.RAW_DATA);
			File affy2 = FileUtil.uploadFile(dc, "/affymetrix/E-TEST-1.ebi.ac.uk", "jos1762.cel.tar.bz2", FileType.RAW_DATA);
			File affy3 = FileUtil.uploadFile(dc, "/affymetrix/E-TEST-1.ebi.ac.uk", "jos1763.cel.tar.bz2", FileType.RAW_DATA);
			dc.commit();
			
			BioSource bioSource = null;
			Sample s1 = null;
			Sample s2 = null;
			Sample sRef = null;
			
			RawBioAssay rba1 = null;
			RawBioAssay rba2 = null;
			RawBioAssay rba1DyeSwap = null;
			RawBioAssay rba2DyeSwap = null;
			RawBioAssay affyRaw1 = null;
			RawBioAssay affyRaw2 = null;
			RawBioAssay affyRaw3 = null;
			
			if (!useBatchImporters)
			{
				dc = TestUtil.getDbControl();
				bioPlate = BioPlate.getById(dc, bioPlate.getId());
				bioSource =  createBioSource(dc, "Biosource A", "medium");
				s1 = createSample(dc, "Sample A.00h", bioSource, 0, hoursUnit);
				s1.setBioWell(bioPlate.getBioWell(0, 0));				
				s2 = createSample(dc, "Sample A.24h", bioSource, 24, hoursUnit);
				s2.setBioWell(bioPlate.getBioWell(0, 1));				
				sRef = createSample(dc, "Sample A.ref", null, null, null);
				sRef.setBioWell(bioPlate.getBioWell(0, 2));
				
				Extract e1 = createExtract(dc, "Extract A.00h", s1);
				e1.setBioWell(bioPlate.getBioWell(1, 0));
				Extract e2 = createExtract(dc, "Extract A.24h", s2);
				e2.setBioWell(bioPlate.getBioWell(1, 1));				
				Extract eRef = createExtract(dc, "Extract A.ref", sRef);
				eRef.setBioWell(bioPlate.getBioWell(1, 2));
				
				Label cy3 = Util.findLabel(dc, "cy3");			
				Label cy5 = Util.findLabel(dc, "cy5");			
				LabeledExtract le1 = createLabeledExtract(dc, "Labeled extract A.00h", e1, cy3);
				le1.setBioWell(bioPlate.getBioWell(2, 0));
				LabeledExtract le1DyeSwap = createLabeledExtract(dc, "Labeled extract A.00h (dye-swap)", e1, cy5);
				le1DyeSwap.setBioWell(bioPlate.getBioWell(3, 0));
				LabeledExtract le2 = createLabeledExtract(dc, "Labeled extract A.24h", e2, cy3);
				le2.setBioWell(bioPlate.getBioWell(2, 1));
				LabeledExtract le2DyeSwap = createLabeledExtract(dc, "Labeled extract A.24h (dye-swap)", e2, cy5);
				le2DyeSwap.setBioWell(bioPlate.getBioWell(3, 1));
				LabeledExtract leRef = createLabeledExtract(dc, "Labeled extract A.ref", eRef, cy5);
				leRef.setBioWell(bioPlate.getBioWell(2, 2));
				LabeledExtract leRefDyeSwap = createLabeledExtract(dc, "Labeled extract A.ref (dye-swap)", eRef, cy3);
				leRefDyeSwap.setBioWell(bioPlate.getBioWell(3, 2));
//				dc.commit();
				
				// Hybridizations, etc.
//				dc = TestUtil.getDbControl();
				Hybridization h1 = createHybridization(dc, "Hybridization A.00h", "Array slide A.1", le1, leRef);
				Hybridization h2 = createHybridization(dc, "Hybridization A.24h", "Array slide A.2", le2, leRef);
				Hybridization h1DyeSwap = createHybridization(dc, "Hybridization A.00h (dye-swap)", "Array slide A.3", le1DyeSwap, leRefDyeSwap);
				Hybridization h2DyeSwap = createHybridization(dc, "Hybridization A.24h (dye-swap)", "Array slide A.4", le2DyeSwap, leRefDyeSwap);
	
				// Affymetrix
				Hybridization affyHyb1 = createHybridization(dc, "Affymetrix hyb A.1", "Affymetrix slide A.1", le1);
				Hybridization affyHyb2 = createHybridization(dc, "Affymetrix hyb A.2", "Affymetrix slide A.2", le2);
				Hybridization affyHyb3 = createHybridization(dc, "Affymetrix hyb A.3", "Affymetrix slide A.3", leRef);
				
				Scan sc1 = createScan(dc, "Scan A.00h", h1);
				Scan sc2 = createScan(dc, "Scan A.24h", h2);
				Scan sc1DyeSwap = createScan(dc, "Scan A.00h (dye-swap)", h1DyeSwap);
				Scan sc2DyeSwap = createScan(dc, "Scan A.24h (dye-swap)", h2DyeSwap);
	
				// Affymetrix
				Scan affyScan1 = createScan(dc, "Affymetrix scan A.1", affyHyb1);
				Scan affyScan2 = createScan(dc, "Affymetrix scan A.2", affyHyb2);
				Scan affyScan3 = createScan(dc, "Affymetrix scan A.3", affyHyb3);
			
				// Images
				// TODO - when we have the image files
				
				// Raw bioassays
				ArrayDesign genericDesign = Util.findArrayDesign(dc, "Array design A");
				ArrayDesign affymetrixDesign = Util.findArrayDesign(dc, "Affymetrix A");
				RawDataType genepix = RawDataTypes.getSafeRawDataType("genepix");
				rba1 = createRawBioAssay(dc, genericPlatform, genepix, "Raw bioassay A.00h", sc1, genericDesign);
				rba2 = createRawBioAssay(dc, genericPlatform, genepix, "Raw bioassay A.24h", sc2, genericDesign);
				rba1DyeSwap = createRawBioAssay(dc, genericPlatform, genepix, "Raw bioassay A.00h (dye-swap)", sc1DyeSwap, genericDesign);
				rba2DyeSwap = createRawBioAssay(dc, genericPlatform, genepix, "Raw bioassay A.24h (dye-swap)", sc2DyeSwap, genericDesign);
				
				// Affymetrix raw bioassays
				affyRaw1 = createRawBioAssay(dc, affymetrixPlatform, null, "Affymetrix raw A.1", affyScan1, affymetrixDesign);
				affyRaw2 = createRawBioAssay(dc, affymetrixPlatform, null, "Affymetrix raw A.2", affyScan2, affymetrixDesign);
				affyRaw3 = createRawBioAssay(dc, affymetrixPlatform, null, "Affymetrix raw A.3", affyScan3, affymetrixDesign);
	
				// Assign Affymetrix CEL files
				FileUtil.addDataFile(dc, affyRaw1, DataFileType.AFFYMETRIX_CEL, affy1, true, true);
				FileUtil.addDataFile(dc, affyRaw2, DataFileType.AFFYMETRIX_CEL, affy2, true, true);
				FileUtil.addDataFile(dc, affyRaw3, DataFileType.AFFYMETRIX_CEL, affy3, true, true);
				dc.commit();
			}
			else
			{
				// Import biomaterials
				dc = TestUtil.getDbControl();
				importBioSources(bioSourceBatchImporter, batchImportFile_BioSource);
				importAnnotations(annotationImporter, batchImportFile_BioSource, Item.BIOSOURCE, getBioSourceColumnHeaders(), "\\t");
				
				importSamples(sampleBatchImporter, batchImportFile_Sample);
				importAnnotations(annotationImporter, batchImportFile_Sample, Item.SAMPLE, getSampleColumnHeaders(), "\\t");
				
				importExtracts(extractBatchImporter, batchImportFile_Extract);
				importLabelExtracts(labeledExtractBatchImporter, batchImportFile_LabeledExtract);
				
				// Hybridizations, etc.
				importHybridizations(hybridizationBatchImporter, batchImportFile_Hybridization);			
				importScans(scanBatchImporter, batchImportFile_Scan);
				importRawBioAssays(rawBioAssayBatchImporter, batchImportFile_RawBioAssay);
				bioSource = Util.findBioSource(dc, "Biosource A");
				s1 = Util.findSample(dc, "Sample A.00h");
				s2 = Util.findSample(dc, "Sample A.24h");
				sRef = Util.findSample(dc, "Sample A.ref");
				
				dc.close();
			}
			dc = TestUtil.getDbControl();
			rba1 = Util.findRawBioAssay(dc, "Raw bioassay A.00h");
			rba2 = Util.findRawBioAssay(dc, "Raw bioassay A.24h");
			rba1DyeSwap = Util.findRawBioAssay(dc, "Raw bioassay A.00h (dye-swap)");
			rba2DyeSwap = Util.findRawBioAssay(dc, "Raw bioassay A.24h (dye-swap)");
			affyRaw1 = Util.findRawBioAssay(dc, "Affymetrix raw A.1");
			affyRaw2 = Util.findRawBioAssay(dc, "Affymetrix raw A.2");
			affyRaw3 = Util.findRawBioAssay(dc, "Affymetrix raw A.3");
			inheritAnnotations(dc, rba1, s1, bioSource);
			inheritAnnotations(dc, rba2, s2, bioSource);
			inheritAnnotations(dc, rba1DyeSwap, s1, bioSource);
			inheritAnnotations(dc, rba2DyeSwap, s2, bioSource);
			
			// Affymetrix
			inheritAnnotations(dc, affyRaw1, s1, bioSource);
			inheritAnnotations(dc, affyRaw2, s2, bioSource);
			inheritAnnotations(dc, affyRaw3, sRef);
			
			PluginConfiguration rawDataImporter = Util.findPluginConfiguration(dc, "Raw data for project A");
			PluginConfiguration rawDataImporterDyeSwap = Util.findPluginConfiguration(dc, "Raw data for project A (dye-swap)");
			dc.commit();
			
			// Import raw data
			importRawData(rawDataImporter, rawData1, rba1);
			importRawData(rawDataImporter, rawData2, rba2);
			importRawData(rawDataImporterDyeSwap, rawData1DyeSwap, rba1DyeSwap);
			importRawData(rawDataImporterDyeSwap, rawData2DyeSwap, rba2DyeSwap);
				
			// TODO - Create spot images - when we have the image files
			
			// Experiment
			dc = TestUtil.getDbControl();
			Experiment exp1 = createExperiment(dc, "Experiment A", 
				RawDataTypes.getSafeRawDataType("genepix"), rba1, rba2, rba1DyeSwap, rba2DyeSwap);
			Experiment affyExperiment = createExperiment(dc, "Affymetrix A", 
				affymetrixPlatform.getRawDataType(), affyRaw1, affyRaw2, affyRaw3);
			dc.commit();
			
			// Experiment overview and Analysis
			AnalysisTest.run("user");
			
			TestUtil.logout();
		}
		catch (Throwable t)
		{
			ok = false;
			t.printStackTrace();
		}
		finally
		{
			if (dc != null) dc.close();
		}
		TestUtil.write("++Leaving user test (" + (ok ? "OK" : "FAILED") + ")\n");
		return ok;
	}

	/**
		Create a biosource.
	*/
	public static BioSource createBioSource(DbControl dc, String name, String drugResistance)
	{
		TestUtil.write("--Creating biosource: " + name + "\n");
		BioSource bioSource = BioSource.getNew(dc);
		bioSource.setName(name);
		dc.saveItem(bioSource);
		
		if (drugResistance != null)
		{
			AnnotationType at = Util.findAnnotationType(dc, "Drug resistance", Item.BIOSOURCE);
			Util.setAnnotation(bioSource, at, drugResistance, null);
		}
		
		return bioSource;
	}
	
	/**
		Create a sample.
	*/
	public static Sample createSample(DbControl dc, String name, BioSource bioSource, Integer time, Unit unit)
	{
		TestUtil.write("--Creating sample: " + name + "\n");
		Sample sample = Sample.getNew(dc);
		sample.setName(name);
		sample.setOriginalQuantity(100.0f);
		sample.setBioSource(bioSource);
		sample.getCreationEvent().setProtocol(Util.findProtocol(dc, "Sampling A"));
		dc.saveItem(sample);
		
		if (time != null)
		{
			AnnotationType at = Util.findAnnotationType(dc, "Time", Item.SAMPLE);
			Util.setAnnotation(sample, at, time, unit);
		}
		
		return sample;
	}
	
	/**
		Create an extract.
	*/
	public static Extract createExtract(DbControl dc, String name, Sample sample)
	{
		TestUtil.write("--Creating extract: " + name + "\n");
		Extract extract = Extract.getNew(dc);
		extract.setName(name);
		extract.setOriginalQuantity(100.0f);
		extract.setSample(sample, 50.0f);
		extract.getCreationEvent().setProtocol(Util.findProtocol(dc, "Extraction A"));
		dc.saveItem(extract);
		return extract;
	}

	/**
		Create a labeled extract.
	*/
	public static LabeledExtract createLabeledExtract(DbControl dc, String name, Extract extract, Label label)
	{
		TestUtil.write("--Creating labeled extract: " + name + "\n");
		
		LabeledExtract labeledExtract = LabeledExtract.getNew(dc, label);
		labeledExtract.setName(name);
		labeledExtract.setOriginalQuantity(100.0f);
		labeledExtract.setExtract(extract, 50.0f);
		labeledExtract.getCreationEvent().setProtocol(Util.findProtocol(dc, "Labeling A"));
		dc.saveItem(labeledExtract);
		return labeledExtract;
	}
	
	/**
	 	Create a bioplate
	 */
	public static BioPlate createBioPlate(DbControl dc, String name, PlateGeometry geometry)
	{
		TestUtil.write("--Creating bioplate: " + name + "\n");
		
		BioPlate plate = BioPlate.getNew(dc, geometry);
		plate.setName(name);
		dc.saveItem(plate);
		return plate;
	}
	
	/**
		Create a hybridization.
	*/
	public static Hybridization createHybridization(DbControl dc, String name, String arraySlideName, LabeledExtract... extracts)
	{
		TestUtil.write("--Creating hybridization: " + name + "\n");
		Hybridization hyb = Hybridization.getNew(dc);
		hyb.setName(name);
		hyb.setArraySlide(Util.findArraySlide(dc, arraySlideName));
		BioMaterialEvent creationEvent = hyb.getCreationEvent();
		creationEvent.setProtocol(Util.findProtocol(dc, "Hybridization A"));
		creationEvent.setHardware(Util.findHardware(dc, "Hybridization station A"));
		for (LabeledExtract le : extracts)
		{
			creationEvent.addSource(le, 50.0f);
		}
		dc.saveItem(hyb);
		return hyb;
	}

	/**
		Create a scan.
	*/
	public static Scan createScan(DbControl dc, String name, Hybridization hyb)
	{
		TestUtil.write("--Creating scan: " + name + "\n");
		Scan scan = Scan.getNew(dc, hyb);
		scan.setName(name);
		scan.setProtocol(Util.findProtocol(dc, "Scanning A"));
		scan.setScanner(Util.findHardware(dc, "Scanner A"));
		dc.saveItem(scan);
		return scan;
	}

	/**
		Create a raw bioassay.
	*/
	public static RawBioAssay createRawBioAssay(DbControl dc, Platform platform, 
		RawDataType rawDataType, String name, Scan scan, ArrayDesign design)
	{
		TestUtil.write("--Creating raw bioassay: " + name + "\n");
		RawBioAssay rba = RawBioAssay.getNew(dc, platform, rawDataType);
		rba.setName(name);
		rba.setScan(scan);
		rba.setProtocol(Util.findProtocol(dc, "Feature extraction A"));
		rba.setSoftware(Util.findSoftware(dc, "Software A"));
		rba.setArrayDesign(design);
		dc.saveItem(rba);
		return rba;
	}
	
	/**
		Inherit all annotations from the parent.
	*/
	public static void inheritAnnotations(DbControl dc, Annotatable item, Annotatable... parents)
	{
		AnnotationSet as = item.getAnnotationSet();
		for (Annotatable parent : parents)
		{
			parent = (Annotatable)parent.getType().getById(dc, parent.getId());
			if (parent.isAnnotated())
			{
				as.inheritAnnotationSet(parent.getAnnotationSet());
			}
		}
	}
	
	/**
		Import features.
	*/
	public static void importRawData(PluginConfiguration config, File file, RawBioAssay rba)
		throws Exception
	{
		TestUtil.write("--Importing raw data from file: " + file.getName() + "\n");
		DbControl dc = TestUtil.getDbControl();
		try
		{
			config = PluginConfiguration.getById(dc, config.getId());
			Job job = Job.getNew(dc, config.getPluginDefinition(), config, null);
			job.setName("Importing data to raw bioassay: " + rba.getName());
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("rawBioAssay", 
				new ItemParameterType<RawBioAssay>(RawBioAssay.class, null), rba);
			job.setParameterValue("missingReporterError", new StringParameterType(), "null");
			job.setParameterValue("featureMismatchError", new StringParameterType(), "smart");
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	/**
		Create an experiment.
	*/
	public static Experiment createExperiment(DbControl dc, String name, RawDataType rawDataType, RawBioAssay... rawBioAssays)
	{
		TestUtil.write("--Creating experiment: " + name + "\n");
		Experiment experiment = Experiment.getNew(dc, rawDataType);
		experiment.setName(name);
		
		for (RawBioAssay rba : rawBioAssays)
		{
			rba = RawBioAssay.getById(dc, rba.getId()); // To get the latest information
			experiment.addRawBioAssay(rba);
			
			AnnotationSet as = rba.getAnnotationSet();
			for (Annotation a : as.getAllInheritedAnnotations().list(dc))
			{
				experiment.addExperimentalFactor(a.getAnnotationType());
			}
			for (Annotation a : as.getAnnotations().list(dc))
			{
				experiment.addExperimentalFactor(a.getAnnotationType());
			}
		}
		
		dc.saveItem(experiment);
		return experiment;
	}

	public static void importRawBioAssays(PluginDefinition batchImporter, File file)
	{
		TestUtil.write("--Importing raw bioassays from file: " + file + "\n");
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PluginDefinition plugin = PluginDefinition.getById(dc, batchImporter.getId());
			Job job = Job.getNew(dc, plugin, null, null);
			
			job.setName("Batch import of raw bioassays to Project A");
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("dataSplitterRegexp", new StringParameterType(), "\\t");
			job.setParameterValue("dataHeaderRegexp", new StringParameterType(), 
					"Name\\tPlatform\\tVariant\\tRaw data type\\tArray index\\tScan\\tSoftware\\tProtocol\\tArray design\\tDescription\\tFile type\\tFile");
			
			job.setParameterValue("nameColumnMapping", new StringParameterType(), "\\Name\\");
			job.setParameterValue("descriptionColumnMapping", new StringParameterType(), "\\Description\\");
			job.setParameterValue("rawDataTypeColumnMapping", new StringParameterType(), "\\Raw data type\\");
			job.setParameterValue("platformColumnMapping", new StringParameterType(), "\\Platform\\");
			job.setParameterValue("variantColumnMapping", new StringParameterType(), "\\Variant\\");
			job.setParameterValue("arrayIndexColumnMapping", new StringParameterType(), "\\Array index\\");
			job.setParameterValue("scanColumnMapping", new StringParameterType(), "\\Scan\\");
			job.setParameterValue("softwareColumnMapping", new StringParameterType(), "\\Software\\");
			job.setParameterValue("protocolColumnMapping", new StringParameterType(), "\\Protocol\\");
			job.setParameterValue("arrayDesignColumnMapping", new StringParameterType(), "\\Array design\\");
			job.setParameterValue("fileTypeColumnMapping", new StringParameterType(), "\\File type\\");
			job.setParameterValue("fileColumnMapping", new StringParameterType(), "\\File\\");
			
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	public static void importScans(PluginDefinition batchImporter, File file)
	{
		TestUtil.write("--Importing scans from file: " + file + "\n");
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PluginDefinition plugin = PluginDefinition.getById(dc, batchImporter.getId());
			Job job = Job.getNew(dc, plugin, null, null);
			
			job.setName("Batch import of scans to Project A");
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("dataSplitterRegexp", new StringParameterType(), "\\t");
			job.setParameterValue("dataHeaderRegexp", new StringParameterType(), 
					"Name\\tHybridization\\tDescription\\tProtocol\\tScanner");
			
			job.setParameterValue("nameColumnMapping", new StringParameterType(), "\\Name\\");
			job.setParameterValue("descriptionColumnMapping", new StringParameterType(), "\\Description\\");
			job.setParameterValue("hybridizationColumnMapping", new StringParameterType(), "\\Hybridization\\");
			job.setParameterValue("hardwareColumnMapping", new StringParameterType(), "\\Scanner\\");			
			job.setParameterValue("protocolColumnMapping", new StringParameterType(), "\\Protocol\\");
			
			
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	public static void importHybridizations(PluginDefinition batchImporter, File file)
	{
		TestUtil.write("--Importing hybridizations from file: " + file + "\n");
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PluginDefinition plugin = PluginDefinition.getById(dc, batchImporter.getId());
			Job job = Job.getNew(dc, plugin, null, null);
			
			job.setName("Batch import of hybridizations to Project A");
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("dataSplitterRegexp", new StringParameterType(), "\\t");
			job.setParameterValue("dataHeaderRegexp", new StringParameterType(), 
					"Name\\tArray slide\\tDescription\\tHardware\\tArrays/slide\\tProtocol\\tParent\\tParent used quantity\\tCreated");
			
			job.setParameterValue("nameColumnMapping", new StringParameterType(), "\\Name\\");
			job.setParameterValue("descriptionColumnMapping", new StringParameterType(), "\\Description\\");
			job.setParameterValue("hardwareColumnMapping", new StringParameterType(), "\\Hardware\\");
			job.setParameterValue("parentColumnMapping", new StringParameterType(), "\\Parent\\");
			job.setParameterValue("usedQuantityColumnMapping", new StringParameterType(), "\\Parent used quantity\\");
			job.setParameterValue("protocolColumnMapping", new StringParameterType(), "\\Protocol\\");
			job.setParameterValue("arraySlideColumnMapping", new StringParameterType(), "\\Array slide\\");
			job.setParameterValue("numArraysColumnMapping", new StringParameterType(), "\\Arrays/slide\\");
			job.setParameterValue("createdColumnMapping", new StringParameterType(), "\\Created\\");
			
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	public static void importLabelExtracts(PluginDefinition batchImporter, File file)
	{
		TestUtil.write("--Importing labeled extracts from file: " + file + "\n");
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PluginDefinition plugin = PluginDefinition.getById(dc, batchImporter.getId());
			Job job = Job.getNew(dc, plugin, null, null);
			
			job.setName("Batch import of labeled extracts to Project A");
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("dataSplitterRegexp", new StringParameterType(), "\\t");
			job.setParameterValue("dataHeaderRegexp", new StringParameterType(), 
					"Name\\tLabel\\tOriginal quantity\\tDescription\\tExternal id\\tProtocol\\tCreated\\tPooled\\tParent\\tParent used quantity\\tBioplate\\tBiowell row\\tBiowell column");
			
			job.setParameterValue("nameColumnMapping", new StringParameterType(), "\\Name\\");
			job.setParameterValue("descriptionColumnMapping", new StringParameterType(), "\\Description\\");
			job.setParameterValue("externalIdColumnMapping", new StringParameterType(), "\\External id\\");
			job.setParameterValue("originalQuantityColumnMapping", new StringParameterType(), "\\Original quantity\\");
			job.setParameterValue("createdColumnMapping", new StringParameterType(), "\\Created\\");
			job.setParameterValue("pooledColumnMapping", new StringParameterType(), "\\Pooled\\");
			job.setParameterValue("parentColumnMapping", new StringParameterType(), "\\Parent\\");
			job.setParameterValue("usedQuantityColumnMapping", new StringParameterType(), "\\Parent used quantity\\");
			job.setParameterValue("protocolColumnMapping", new StringParameterType(), "\\Protocol\\");
			job.setParameterValue("labelColumnMapping", new StringParameterType(), "\\Label\\");
			job.setParameterValue("bioPlateColumnMapping", new StringParameterType(), "\\Bioplate\\");
			job.setParameterValue("bioWellRowColumnMapping", new StringParameterType(), "\\Biowell row\\");
			job.setParameterValue("bioWellColColumnMapping", new StringParameterType(), "\\Biowell column\\");
			
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
		}
		finally
		{
			if (dc != null) dc.close();
		}		
	}

	public  static void importExtracts(PluginDefinition batchImporter,	File file)
	{
		TestUtil.write("--Importing extracts from file: " + file + "\n");
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PluginDefinition plugin = PluginDefinition.getById(dc, batchImporter.getId());
			Job job = Job.getNew(dc, plugin, null, null);
			
			job.setName("Batch import of extracts to Project A");
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("dataSplitterRegexp", new StringParameterType(), "\\t");
			job.setParameterValue("dataHeaderRegexp", new StringParameterType(), 
					"Name\\tOriginal quantity\\tDescription\\tExternal id\\tProtocol\\tCreated\\tPooled\\tParent\\tParent used quantity\\tBioplate\\tBiowell row\\tBiowell column");
			
			job.setParameterValue("nameColumnMapping", new StringParameterType(), "\\Name\\");
			job.setParameterValue("descriptionColumnMapping", new StringParameterType(), "\\Description\\");
			job.setParameterValue("externalIdColumnMapping", new StringParameterType(), "\\External id\\");
			job.setParameterValue("originalQuantityColumnMapping", new StringParameterType(), "\\Original quantity\\");
			job.setParameterValue("createdColumnMapping", new StringParameterType(), "\\Created\\");
			job.setParameterValue("pooledColumnMapping", new StringParameterType(), "\\Pooled\\");
			job.setParameterValue("parentColumnMapping", new StringParameterType(), "\\Parent\\");
			job.setParameterValue("usedQuantityColumnMapping", new StringParameterType(), "\\Parent used quantity\\");
			job.setParameterValue("protocolColumnMapping", new StringParameterType(), "\\Protocol\\");
			job.setParameterValue("bioPlateColumnMapping", new StringParameterType(), "\\Bioplate\\");
			job.setParameterValue("bioWellRowColumnMapping", new StringParameterType(), "\\Biowell row\\");
			job.setParameterValue("bioWellColColumnMapping", new StringParameterType(), "\\Biowell column\\");
			
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	public static void importSamples(PluginDefinition batchImporter, File file)
	{
		TestUtil.write("--Importing samples from file: " + file + "\n");
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PluginDefinition plugin = PluginDefinition.getById(dc, batchImporter.getId());
			Job job = Job.getNew(dc, plugin, null, null);
			
			job.setName("Batch import of samples to Project A");
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("dataSplitterRegexp", new StringParameterType(), "\\t");
			job.setParameterValue("dataHeaderRegexp", new StringParameterType(), 
					"Name\\tOriginal quantity\\tDescription\\tExternal id\\tCreated\\tPooled\\tBiosource\\tProtocol\\tBioplate\\tBiowell row\\tBiowell column\\tTime");
			
			job.setParameterValue("nameColumnMapping", new StringParameterType(), "\\Name\\");
			job.setParameterValue("descriptionColumnMapping", new StringParameterType(), "\\Description\\");
			job.setParameterValue("externalIdColumnMapping", new StringParameterType(), "\\External id\\");
			job.setParameterValue("originalQuantityColumnMapping", new StringParameterType(), "\\Original quantity\\");
			job.setParameterValue("createdColumnMapping", new StringParameterType(), "\\Created\\");
			job.setParameterValue("pooledColumnMapping", new StringParameterType(), "\\Pooled\\");
			job.setParameterValue("parentColumnMapping", new StringParameterType(), "\\Biosource\\");
			job.setParameterValue("protocolColumnMapping", new StringParameterType(), "\\Protocol\\");
			job.setParameterValue("bioPlateColumnMapping", new StringParameterType(), "\\Bioplate\\");
			job.setParameterValue("bioWellRowColumnMapping", new StringParameterType(), "\\Biowell row\\");
			job.setParameterValue("bioWellColColumnMapping", new StringParameterType(), "\\Biowell column\\");
			
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
		}
		finally
		{
			if(dc != null) dc.close();
		}		
	}

	public static void importBioSources(PluginDefinition batchImporter, File file)
	{
		TestUtil.write("--Importing biosources from file: " + file + "\n");
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PluginDefinition plugin = PluginDefinition.getById(dc, batchImporter.getId());
			Job job = Job.getNew(dc, plugin, null, null);
			
			job.setName("Batch import of bio-sources to Project A");
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("dataSplitterRegexp", new StringParameterType(), "\\t");
			job.setParameterValue("dataHeaderRegexp", new StringParameterType(), "Name\\tDescription\\tExternal id\\tDrug resistance");
			
			job.setParameterValue("nameColumnMapping", new StringParameterType(), "\\Name\\");
			job.setParameterValue("descriptionColumnMapping", new StringParameterType(), "\\Description\\");
			job.setParameterValue("externalIdColumnMapping", new StringParameterType(), "\\External id\\");
			
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
		}
		finally
		{
			if(dc != null) dc.close();
		}		
	}
	
	public static void importAnnotations(PluginDefinition annotationImporter, File file, Item item, String[] columnHeaders, String dataSplitter)
	{
		TestUtil.write("--Importing annotations for " + item.name() + " from file: " + file + "\n");
		DbControl dc = null;
		StringBuilder headerString = new StringBuilder();
		try
		{
			dc = TestUtil.getDbControl();
			PluginDefinition plugin = PluginDefinition.getById(dc, annotationImporter.getId());
			Job job = Job.getNew(dc, plugin, null, null);
			job.setName("Batch import of annotations for " + item.name());
			
			// Get annotation types for the current item
			ItemQuery<AnnotationType> query = AnnotationType.getQuery(item);
			query.include(Include.MINE, Include.SHARED, Include.IN_PROJECT, Include.OTHERS);
			query.order(Orders.asc(Hql.property("name")));
			List<AnnotationType> annotationTypes;
			Map<String, AnnotationType> names = 
				new TreeMap<String, AnnotationType>(String.CASE_INSENSITIVE_ORDER);
			
			annotationTypes = new ArrayList<AnnotationType>(query.list(dc));
			for (AnnotationType at : annotationTypes)
			{
				names.put(at.getName(), at);
			}
			for (int i=0; i<columnHeaders.length; i++)
			{
				String header = columnHeaders[i];
				headerString.append(i>0 ? "\\t" : "");
				headerString.append(header);
				if (names.get(header) != null)
				{
					job.setParameterValue("columnMapping." + i, 
							new ItemParameterType<AnnotationType>(AnnotationType.class, annotationTypes), 
							names.get(header));
				}
			}
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("dataSplitterRegexp", new StringParameterType(), dataSplitter);
			job.setParameterValue("dataHeaderRegexp", new StringParameterType(), headerString.toString());
			job.setParameterValue("nameColumnMapping", new StringParameterType(), "\\Name\\");
			job.setParameterValue("itemType", new StringParameterType(), item.name());
			
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
		}
		finally
		{
			if (dc != null ) dc.close();
		}
	}
	
	private static String[] getBioSourceColumnHeaders()
	{
		return new String[] {"Name", "Description", "External id", "Drug resistance"};
	}
	
	private static String[] getSampleColumnHeaders()
	{
		return new String[] {"Name", "Original quantity", "Description", "External id", "Created", "Pooled", "Biosource", "Protocol", "Bioplate", "Biowell row", "Biowell column", "Time"};
	}
}
