/* $Id: table.js 4889 2009-04-06 12:52:39Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson, Gregory Vincic
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	JavaScript functions for the Table taglib.

	@author Nicklas
	@version 2.0
*/
var Table = new TableClass();
function TableClass()
{
	this.tables = new Array();

	/**
		Add a column to the table
		@param tableId The ID of the table to add the column to
		@param column The Column object to add
	*/
	this.addColumn = function(tableId, column)
	{
		var columns = this.tables[tableId];
		if (!columns)
		{
			columns = new Array();
			this.tables[tableId] = columns;
		}
		columns[columns.length] = column;
		columns['id'+column.id] = column;
		if (column.property)
		{
			columns['prop'+column.property] = column;
		}
	}
	
	/**
		Get all columns defined for the specified table.
		@param tableId The ID of the table to get the columns for
	*/
	this.getColumnDefs = function(tableId)
	{
		var columns = this.tables[tableId];
		return this.tables[tableId];
	}

	/**
		Change the sort order of the table.
		@param tableId The ID attribute of the table tag
		@param sortby The new sortby value
		@param direction The new direction value
	*/
	this.setSortOrder = function(evt, tableId, sortby, direction)
	{
		var frm = document.forms[tableId];
		if (frm.sortby.value != sortby || frm.direction.value != direction)
		{
			if (evt.altKey || evt.ctrlKey || evt.shiftKey) 
			{
				var oldSortby = frm.sortby.value.split(',');
				var newSortby = new Array();
				for (var i = 0; i < oldSortby.length; i++)
				{
					if (oldSortby[i] != sortby) newSortby[newSortby.length] = oldSortby[i];
				}
				newSortby[newSortby.length] = sortby;
				sortby = newSortby.join(',');
			}
			frm.sortby.value = sortby;
			frm.direction.value = direction;
			frm.submit();
		}
	}

	/**
		Change the page of the table.
		@param tableId The ID attribute of the table tag
		@param page The new page
	*/
	this.setPage = function(tableId, page)
	{
		var frm = document.forms[tableId];
		if (frm.page.value != page)
		{
			frm.page.value = page;
			frm.submit();
		}
	}
	
	/**
		Change the visible columns of the table.
		@param tableId The ID attribute of the table tag
		@param columns Comma-separated list of column ID:s of the visible columns, or the value 'all'
	*/
	this.setColumns = function(tableId, columns)
	{
		var frm = document.forms[tableId];
		if (frm.columns.value != columns)
		{
			frm.columns.value = columns;
			frm.submit();
		}
	}

	/**
		Show columns in the table, preserving the currently visible columns.
		@param tableId The ID attribute of the table tag
		@param columns Comma-separated list of column ID:s of the visible columns, or the value 'all'
	*/
	this.showColumns = function(tableId, columns)
	{
		var frm = document.forms[tableId];
		if (frm.columns.value != columns)
		{
			frm.columns.value += ',' + columns;
			frm.submit();
		}
	}
	
	/**
		Change one of the checkbox filtering options of the table.
		@param tableId The ID attribute of the table tag
		@param name The name of the option
		@param checked If the option is checked or uncheck
	*/
	this.setOption = function(tableId, name, checked)
	{
		var frm = document.forms[tableId];
		frm[name].value = checked;
		frm.submit();
	}

	/**
		Change one of the checkbox filtering options of the table.
		@param tableId The ID attribute of the table tag
		@param name The name of the option
		@param checked If the option is checked or uncheck
	*/
	this.setFilter = function(tableId, filterId)
	{
		var frm = document.forms[tableId];
		frm.filter_id.value = filterId;
		frm.submit();
	}

	/**
		Get the visible columns of the table.
		@param tableId The ID attribute of the table tag
	*/
	this.getColumns = function(tableId)
	{
		var frm = document.forms[tableId];
		return frm.columns.value;
	}
	
	/**
		Open a popup window that allows a user to configure which columns
		that should be visible and their order.
		@param ID The SessionControl ID
		@param tableId The ID attribute of the table tag
		@param itemType The Item.name() value of the type of items in the list
	*/
	this.configureColumns = function(ID, tableId, itemType, defaultColumns, subContext)
	{
		var url = getRoot()+'common/columns/configure.jsp?ID='+ID+'&table_id='+tableId+'&item_type='+itemType;
		if (defaultColumns) url += '&default='+defaultColumns;
		if (subContext) url += '&subcontext='+subContext;
		Main.openPopup(url, 'ConfigureColumns', 800, 400);
	}

	/**
		Open a popup window that allows a user to configure a filter for the list.
		@param ID The SessionControl ID
		@param tableId The ID attribute of the table tag
		@param itemType The Item.name() value of the type of items in the list
	*/
	this.configureFilter = function(ID, tableId, itemType)
	{
		Main.openPopup(getRoot()+'common/filter/configure.jsp?ID='+ID+'&table_id='+tableId+'&item_type='+itemType, 'ConfigureFilter', 500, 400);
	}
	
	this.submitToPopup = function(tableId, cmd, width, height)
	{
		var target = tableId+cmd;
		Main.openPopup('', target, width, height);
		var frm = document.forms[tableId];
		var oldCmd = frm.cmd.value;
		frm.cmd.value = cmd;
		frm.target = target;
		frm.submit();
		frm.target = window.name;
		frm.cmd.value = oldCmd;
	}
	
	this.configureExport = function(submitPage, ID, tableId, itemType, defaultColumns, extraParameters)
	{
		var url = submitPage+'?ID='+ID+'&cmd=Export&item_type='+itemType+'&table_id='+tableId
		if (defaultColumns) url += '&default='+defaultColumns;
		if (extraParameters) url += '&'+extraParameters;
		Main.openPopup(url, 'Export'+tableId, 600, 460);
	}
	
	this.poolItems = function(submitPage, ID, tableId, itemType, cmd, regexp)
	{
		var frm = document.forms[tableId];
		if (Forms.numChecked(frm) < 1)
		{
			alert('Please select at least one item in the list');
			return;
		}
		var oldAction = frm.action;
		var oldCmd = frm.cmd.value;
		frm.action = submitPage;
		frm.cmd.value = cmd ? cmd : "NewPooledItem";
		frm.target = 'NewPooledItem'+tableId;
		Main.openPopup('', 'NewPooledItem'+tableId, 800, 500);
		frm.submit();
		frm.target = window.name;
		frm.action = oldAction;
		frm.cmd.value = oldCmd;
	}
	
	this.setOwnerOfItems = function(submitPage, ID, tableId, itemType, cmd, regexp)
	{
		var frm = document.forms[tableId];
		if (Forms.numChecked(frm, regexp) == 0)
		{
			alert('Please select at least one item in the list');
			return; 
		}
		var oldAction = frm.action;
		var oldCmd = frm.cmd.value;
		frm.action = submitPage;
		frm.cmd.value = cmd ? cmd : "SetOwner";
		frm.target = 'SetOwner'+tableId;
		Main.openPopup('', 'SetOwner'+tableId, 450, 150);
		frm.submit();
		frm.target = window.name;
		frm.action = oldAction;
		frm.cmd.value = oldCmd;
	} 
	
	this.shareItems = function(submitPage, ID, tableId, itemType, cmd, regexp)
	{
		var frm = document.forms[tableId];
		if (Forms.numChecked(frm, regexp) == 0)
		{
			alert('Please select at least one item in the list');
			return; 
		}
		var oldAction = frm.action;
		var oldCmd = frm.cmd.value;
		frm.action = submitPage;
		frm.cmd.value = cmd ? cmd : "Share";
		frm.target = 'Share'+tableId;
		Main.openPopup('', 'Share'+tableId, 600, 400);
		frm.submit();
		frm.target = window.name;
		frm.action = oldAction;
		frm.cmd.value = oldCmd;
	}
	
	this.loadContext = function(tableId, name)
	{
		var frm = document.forms[tableId];
		Forms.createHidden(frm, 'context', name);
		frm.cmd.value = 'LoadContext';
		frm.submit();
	}
	
	this.clearFilter = function(tableId)
	{
		var frm = document.forms[formId];
		Forms.createHidden(frm, 'filter:clearAll', '1');
		frm.submit();
	}
	
	this.saveCurrentContext = function(ID, itemType, subContext)
	{
		var url = getRoot()+'common/context/saveas.jsp?ID='+ID+'&item_type='+itemType+'&subcontext='+subContext;
		Main.openPopup(url, 'SaveCurrentContext', 450, 280);
	}

	this.manageContexts = function(ID, itemType, subContext)
	{
		var url = getRoot()+'common/context/manage.jsp?ID='+ID+'&item_type='+itemType+'&subcontext='+subContext;
		Main.openPopup(url, 'ManageContexts', 450, 320);
	}

	this.presetOnChange = function(ID, tableId, itemType, defaultColumns, subContext)
	{
		var frm = document.forms[tableId];
		var action = frm.presetselector[frm.presetselector.selectedIndex].value;
		frm.presetselector.selectedIndex = 0;
		if (action.substr(0, 7) == 'saveas:')
		{
			this.saveCurrentContext(ID, itemType, action.substr(7));
		}
		else if (action.substr(0, 7) == 'manage:')
		{
			this.manageContexts(ID, itemType, action.substr(7));
		}
		else if (action.substr(0, 5) == 'load:')
		{
			this.loadContext(tableId, action.substr(5));
		}
		else if (action == 'managecolumns:')
		{
			this.configureColumns(ID, tableId, itemType, defaultColumns, subContext);
		}
		else if (action == 'clearfilter:')
		{
			this.clearFilter(tableId)
		}
		else if (action.substr(0, 7) == 'option:')
		{
			var colon = action.indexOf(':', 7);
			var option = action.substring(7, colon);
			var value = action.substr(colon+1);
			this.setOption(tableId, option, value);
		}
	}

	
	/**
		Return the selected items to the parent using a 
		callback function.
		@param tableId The ID attribute of the table tag
		@param callback A function object which should take two parameters:
			the ID and title of the selected item
		@return The number of selected items
	*/
	this.returnSelected = function(tableId, callback)
	{
		if (!callback) return 0;
		var frm = document.forms[tableId];
		var numSelected = 0;
		if (frm.item_id)
		{
			var element = Forms.getCheckedRadio(frm.item_id);
			if (element != null)
			{
				callback(element.value, element.title);
				numSelected = 1;
			}
		}
		else
		{
			for (var i=0; i < frm.elements.length; i++) // >
			{
				var element = frm.elements[i];
				if (element.type == 'checkbox' && element.name.match(/\d+/) && element.checked)
				{
					callback(element.name, element.title);
					numSelected++;
				}
			}
		}
		return numSelected;
	}
	
	/**
		Get the ID of all selected items.
		@param tableId The ID attribute of the table tag
		@return An array object with the ID:s
	*/
	this.getSelected = function(tableId)
	{
		var frm = document.forms[tableId];
		var items = new Array();
		if (frm.item_id)
		{
			var element = Forms.getCheckedRadio(frm.item_id);
			if (element != null)
			{
				items[items.length] = element.value;
			}
		}
		else
		{
			for (var i=0; i < frm.elements.length; i++) // >
			{
				var element = frm.elements[i];
				if (element.type == 'checkbox' && element.name.match(/\d+/) && element.checked)
				{
					items[items.length] = element.name;
				}
			}
		}
		return items;
	}
	
	/**
		Initialise the columns list of the quick filter
		@param tableId The ID attribute of the table tag
		@param property The filter property value of the currently selected column
		@param value The current filter value
	*/
	this.initQuickFilter = function(tableId, property, value)
	{
		var frm = document.forms[tableId];
		var quickcolumn = frm.quickcolumn;
		var selectedIndex = 0;
		frm.quickvalue.value = value;
		for (var i = 0; i < this.columns.length; i++)
		{
			var col = this.columns[i];
			if (col.dataType && col.visible)
			{
				selectedIndex = col.filterProperty == property ? quickcolumn.length : selectedIndex;
				quickcolumn[quickcolumn.length] = new Option(col.title, col.filterProperty, false, false);
			}
		}
		quickcolumn.selectedIndex = selectedIndex;
		if (!this.columns['prop'+property] && quickcolumn.length > 0) property = quickcolumn[0].value;
		this.quickColumnOnChange(tableId, property, value);
	}
	
	var oldType;
	// Update the displayed filter depending on the dataType of the selected column
	this.quickColumnOnChange = function(tableId, property, value)
	{
		var column = this.columns['prop'+property];
		var dataType = column.dataType;
		var frm = document.forms[tableId];
		frm.quickcolumn_type.value = dataType;
		frm.quickcolumn_collection.value = column.isCollection ? 1 : 0;
		if (oldType) Main.hide(oldType);
		if (column.enumeration)
		{
			frm.quickvalue_enum.length = 0;
			frm.quickvalue_enum[frm.quickvalue_enum.length] = new Option();
			for (var i = 0; i < column.enumeration.length; i++) // >
			{
				frm.quickvalue_enum[frm.quickvalue_enum.length] = new Option(column.enumeration[i].text, column.enumeration[i].value);
			}
			if (value != undefined) Forms.selectListOption(frm.quickvalue_enum, value);
			oldType = 'enum';
		}
		else
		{
			if (dataType == 'string')
			{
				oldType = 'string';
				if (value != undefined) frm.quickvalue_string.value = value;
			}
			else if (dataType == 'int')
			{
				oldType = 'int';
				if (value != undefined) frm.quickvalue_int.value = value;
			}
			else if (dataType == 'long')
			{
				oldType = 'long';
				if (value != undefined) frm.quickvalue_long.value = value;
			}
			else if (dataType == 'float')
			{
				oldType = 'float';
				if (value != undefined) frm.quickvalue_float.value = value;
			}
			else if (dataType == 'double')
			{
				oldType = 'double';
				if (value != undefined) frm.quickvalue_double.value = value;
			}
			else if (dataType == 'date')
			{
				oldType = 'date';
				if (value != undefined) frm.quickvalue_date.value = value;
			}
			else if (dataType == 'boolean')
			{
				oldType = 'boolean';
				Forms.checkRadio(frm.quickvalue_boolean, value);
			}
		}
		Main.showInline(oldType);
	}
	
	this.setQuickValue = function(frm, value)
	{
		frm.quickvalue.value = value;
		frm.submit();
	}
	
	this.itemOnClick = function(tableId, evt, itemId, mode, viewFunction, editFunction, returnFunction)
	{
		var frm = document.forms[tableId];
		if (mode == 'selectone')
		{
			Forms.checkRadio(frm.item_id, itemId);
			returnFunction();
		}
		else if (mode == 'selectmultiple' || mode == 'selectmultiplenobuttons')
		{
			this.toggleCheckBox(tableId, itemId);
		}
		else
		{
			if (evt && (evt.altKey || evt.ctrlKey || evt.shiftKey))
			{
				editFunction(itemId);
			}
			else
			{
				viewFunction(itemId);
			}
		}
	}
	
	this.toggleCheckBox = function(tableId, itemId)
	{
		var frm = document.forms[tableId];
		for (var i=0; i < frm.elements.length; i++) // >
		{
			var element = frm.elements[i];
			if (element.type == 'checkbox' && element.name == itemId)
			{
				element.checked = !element.checked;
			}
		}
	}

}

function Column(id, title, property, sortProperty, filterProperty, exportProperty, dataType, isCollection, isAnnotation, visible, alwaysShow, alwaysHide, sortable, filterable, exportable, enumeration)
{
	this.id = id;
	this.title = title;
	this.property = property;
	this.sortProperty = sortProperty;
	this.filterProperty = filterProperty;
	this.exportProperty = exportProperty;
	this.dataType = dataType;
	this.isCollection = isCollection;
	this.isAnnotation = isAnnotation;
	this.visible = visible;
	this.alwaysShow = alwaysShow;
	this.alwaysHide = alwaysHide;
	this.sortable = sortable;
	this.filterable = filterable;
	this.exportable = exportable;
	this.enumeration = enumeration;
}


